
/* GENERATED CODE... DO NOT EDIT */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * Copyright (C) 2012 Lanedo GmbH
 * Copyright (C) 2012-2017 Aleksander Morgado <aleksander@aleksander.es>
 */


#include <string.h>

#include "qmi-voice.h"
#include "qmi-enum-types.h"
#include "qmi-enum-types-private.h"
#include "qmi-flags64-types.h"
#include "qmi-error-types.h"
#include "qmi-device.h"
#include "qmi-utils.h"

#define QMI_STATUS_SUCCESS 0x0000
#define QMI_STATUS_FAILURE 0x0001


typedef enum {
    QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES = 0x001E,
    QMI_MESSAGE_VOICE_DIAL_CALL = 0x0020,
    QMI_MESSAGE_VOICE_END_CALL = 0x0021,
    QMI_MESSAGE_VOICE_ANSWER_CALL = 0x0022,
    QMI_MESSAGE_VOICE_GET_CONFIG = 0x0041,
} QmiMessageVoice;


typedef enum {
    QMI_INDICATION_VOICE_ALL_CALL_STATUS = 0x002E,
} QmiIndicationVoice;


/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message VOICE Get Supported Messages */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_voice_get_supported_messages_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_VOICE,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES);

    return self;
}

/* --- Output -- */

/**
 * QmiMessageResult:
 * @error_status: a #guint16.
 * @error_code: a #guint16.
 *
 * A QmiMessageResult struct.
 *
 * Since: 1.14
 */
typedef struct _QmiMessageResult {
    guint16 error_status;
    guint16 error_code;
} QmiMessageResult;

struct _QmiMessageVoiceGetSupportedMessagesOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* List */
    gboolean arg_list_set;
    GArray *arg_list;
};

#define QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST 0x10

gboolean
qmi_message_voice_get_supported_messages_output_get_result (
    QmiMessageVoiceGetSupportedMessagesOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

gboolean
qmi_message_voice_get_supported_messages_output_get_list (
    QmiMessageVoiceGetSupportedMessagesOutput *self,
    GArray **value_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'List' was not found in the message");
        return FALSE;
    }

    if (value_list)
        *value_list = self->arg_list;

    return TRUE;
}

GType
qmi_message_voice_get_supported_messages_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceGetSupportedMessagesOutput"),
                                          (GBoxedCopyFunc) qmi_message_voice_get_supported_messages_output_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_get_supported_messages_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceGetSupportedMessagesOutput *
qmi_message_voice_get_supported_messages_output_ref (QmiMessageVoiceGetSupportedMessagesOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_get_supported_messages_output_unref (QmiMessageVoiceGetSupportedMessagesOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_list)
            g_array_unref (self->arg_list);
        g_slice_free (QmiMessageVoiceGetSupportedMessagesOutput, self);
    }
}

static gboolean
qmi_message_result_validate (
    const guint8 *buffer,
    guint16 buffer_len)
{
    static const guint expected_len = 4;

    if (buffer_len < expected_len) {
        g_warning ("Cannot read the 'Result' TLV: expected '%u' bytes, but only got '%u' bytes",
                   expected_len, buffer_len);
        return FALSE;
    }

    return TRUE;
}

static gchar *
qmi_message_result_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    guint16 error_status;
    guint16 error_code;

    if ((init_offset = qmi_message_tlv_read_init (self, QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT, NULL, NULL)) == 0)
        return NULL;
    if (!qmi_message_tlv_read_guint16 (self, init_offset, &offset, QMI_ENDIAN_LITTLE, &error_status, NULL))
        return NULL;
    if (!qmi_message_tlv_read_guint16 (self, init_offset, &offset, QMI_ENDIAN_LITTLE, &error_code, NULL))
        return NULL;
    g_warn_if_fail (__qmi_message_tlv_read_remaining_size (self, init_offset, offset) == 0);

    if (error_status == QMI_STATUS_SUCCESS)
        return g_strdup ("SUCCESS");

    return g_strdup_printf ("FAILURE: %s", qmi_protocol_error_get_string ((QmiProtocolError) error_code));
}

static gchar *
qmi_message_voice_get_supported_messages_output_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint list_i;
        guint16 list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (list_i = 0; list_i < list_n_items; list_i++) {
            g_string_append_printf (printable, " [%u] = '", list_i);

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%u", (guint)tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_supported_messages_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_supported_messages_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_supported_messages_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST:
            tlv_type_str = "List";
            translated_value = qmi_message_voice_get_supported_messages_output_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_supported_messages_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Supported Messages\" (0x001E)\n",
                            line_prefix);

    {
        struct message_get_supported_messages_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_supported_messages_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageVoiceGetSupportedMessagesOutput *
__qmi_message_voice_get_supported_messages_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageVoiceGetSupportedMessagesOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES);

    self = g_slice_new0 (QmiMessageVoiceGetSupportedMessagesOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_voice_get_supported_messages_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_voice_get_supported_messages_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_supported_messages_output_list_out;
            }
            {
                guint list_i;
                guint16 list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(list_n_items), NULL))
                    goto qmi_message_voice_get_supported_messages_output_list_out;

                self->arg_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint8),
                    (guint)list_n_items);

                for (list_i = 0; list_i < list_n_items; list_i++) {
                    guint8 list_aux;

                    if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(list_aux), NULL))
                        goto qmi_message_voice_get_supported_messages_output_list_out;
                    g_array_insert_val (self->arg_list, list_i, list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'List' TLV", offset);
            }

            self->arg_list_set = TRUE;

qmi_message_voice_get_supported_messages_output_list_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message VOICE Dial Call */


/* --- Input -- */

struct _QmiMessageVoiceDialCallInput {
    volatile gint ref_count;

    /* Calling Number */
    gboolean arg_calling_number_set;
    gchar *arg_calling_number;
};

#define QMI_MESSAGE_VOICE_DIAL_CALL_INPUT_TLV_CALLING_NUMBER 0x01

gboolean
qmi_message_voice_dial_call_input_get_calling_number (
    QmiMessageVoiceDialCallInput *self,
    const gchar **value_calling_number,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_calling_number_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Calling Number' was not found in the message");
        return FALSE;
    }

    if (value_calling_number)
        *value_calling_number = self->arg_calling_number;

    return TRUE;
}

gboolean
qmi_message_voice_dial_call_input_set_calling_number (
    QmiMessageVoiceDialCallInput *self,
    const gchar *value_calling_number,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    g_free (self->arg_calling_number);
    self->arg_calling_number = g_strdup (value_calling_number ? value_calling_number : "");
    self->arg_calling_number_set = TRUE;

    return TRUE;
}

GType
qmi_message_voice_dial_call_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceDialCallInput"),
                                          (GBoxedCopyFunc) qmi_message_voice_dial_call_input_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_dial_call_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceDialCallInput *
qmi_message_voice_dial_call_input_ref (QmiMessageVoiceDialCallInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_dial_call_input_unref (QmiMessageVoiceDialCallInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_free (self->arg_calling_number);
        g_slice_free (QmiMessageVoiceDialCallInput, self);
    }
}

QmiMessageVoiceDialCallInput *
qmi_message_voice_dial_call_input_new (void)
{
    QmiMessageVoiceDialCallInput *self;

    self = g_slice_new0 (QmiMessageVoiceDialCallInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_voice_dial_call_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageVoiceDialCallInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_VOICE,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_VOICE_DIAL_CALL);

    /* There is at least one mandatory TLV, don't allow NULL input */
    if (!input) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Message 'Dial Call' has mandatory TLVs");
        goto error_out;
    }

    /* Try to add the 'Calling Number' TLV */
    if (input->arg_calling_number_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_DIAL_CALL_INPUT_TLV_CALLING_NUMBER, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Calling Number': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_string (self, 0, input->arg_calling_number, -1, error)) {
            g_prefix_error (error, "Cannot write string in TLV 'Calling Number': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Calling Number': ");
            goto error_out;
        }
    } else {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Missing mandatory TLV 'Calling Number' in message 'Dial Call'");
        goto error_out;
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageVoiceDialCallOutput {
    volatile gint ref_count;

    /* Call ID */
    gboolean arg_call_id_set;
    guint8 arg_call_id;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_CALL_ID 0x10
#define QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_RESULT 0x02

gboolean
qmi_message_voice_dial_call_output_get_call_id (
    QmiMessageVoiceDialCallOutput *self,
    guint8 *value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call ID' was not found in the message");
        return FALSE;
    }

    if (value_call_id)
        *value_call_id = self->arg_call_id;

    return TRUE;
}

gboolean
qmi_message_voice_dial_call_output_get_result (
    QmiMessageVoiceDialCallOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_voice_dial_call_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceDialCallOutput"),
                                          (GBoxedCopyFunc) qmi_message_voice_dial_call_output_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_dial_call_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceDialCallOutput *
qmi_message_voice_dial_call_output_ref (QmiMessageVoiceDialCallOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_dial_call_output_unref (QmiMessageVoiceDialCallOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceDialCallOutput, self);
    }
}

static gchar *
qmi_message_voice_dial_call_input_calling_number_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_DIAL_CALL_INPUT_TLV_CALLING_NUMBER, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_dial_call_output_call_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_dial_call_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_dial_call_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_dial_call_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_VOICE_DIAL_CALL_INPUT_TLV_CALLING_NUMBER:
            tlv_type_str = "Calling Number";
            translated_value = qmi_message_voice_dial_call_input_calling_number_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_CALL_ID:
            tlv_type_str = "Call ID";
            translated_value = qmi_message_voice_dial_call_output_call_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_dial_call_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Dial Call\" (0x0020)\n",
                            line_prefix);

    {
        struct message_dial_call_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_dial_call_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageVoiceDialCallOutput *
__qmi_message_voice_dial_call_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageVoiceDialCallOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_MESSAGE_VOICE_DIAL_CALL);

    self = g_slice_new0 (QmiMessageVoiceDialCallOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0) {
                goto qmi_message_voice_dial_call_output_call_id_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_call_id), NULL))
                goto qmi_message_voice_dial_call_output_call_id_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call ID' TLV", offset);
            }

            self->arg_call_id_set = TRUE;

qmi_message_voice_dial_call_output_call_id_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_DIAL_CALL_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_voice_dial_call_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_voice_dial_call_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message VOICE End Call */


/* --- Input -- */

struct _QmiMessageVoiceEndCallInput {
    volatile gint ref_count;

    /* Call ID */
    gboolean arg_call_id_set;
    guint8 arg_call_id;
};

#define QMI_MESSAGE_VOICE_END_CALL_INPUT_TLV_CALL_ID 0x01

gboolean
qmi_message_voice_end_call_input_get_call_id (
    QmiMessageVoiceEndCallInput *self,
    guint8 *value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call ID' was not found in the message");
        return FALSE;
    }

    if (value_call_id)
        *value_call_id = self->arg_call_id;

    return TRUE;
}

gboolean
qmi_message_voice_end_call_input_set_call_id (
    QmiMessageVoiceEndCallInput *self,
    guint8 value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_call_id = value_call_id;
    self->arg_call_id_set = TRUE;

    return TRUE;
}

GType
qmi_message_voice_end_call_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceEndCallInput"),
                                          (GBoxedCopyFunc) qmi_message_voice_end_call_input_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_end_call_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceEndCallInput *
qmi_message_voice_end_call_input_ref (QmiMessageVoiceEndCallInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_end_call_input_unref (QmiMessageVoiceEndCallInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceEndCallInput, self);
    }
}

QmiMessageVoiceEndCallInput *
qmi_message_voice_end_call_input_new (void)
{
    QmiMessageVoiceEndCallInput *self;

    self = g_slice_new0 (QmiMessageVoiceEndCallInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_voice_end_call_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageVoiceEndCallInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_VOICE,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_VOICE_END_CALL);

    /* There is at least one mandatory TLV, don't allow NULL input */
    if (!input) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Message 'End Call' has mandatory TLVs");
        goto error_out;
    }

    /* Try to add the 'Call ID' TLV */
    if (input->arg_call_id_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_END_CALL_INPUT_TLV_CALL_ID, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Call ID': ");
            goto error_out;
        }

        /* Write the guint8 variable to the buffer */
        if (!qmi_message_tlv_write_guint8 (self, input->arg_call_id, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Call ID': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Call ID': ");
            goto error_out;
        }
    } else {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Missing mandatory TLV 'Call ID' in message 'End Call'");
        goto error_out;
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageVoiceEndCallOutput {
    volatile gint ref_count;

    /* Call ID */
    gboolean arg_call_id_set;
    guint8 arg_call_id;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_CALL_ID 0x10
#define QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_RESULT 0x02

gboolean
qmi_message_voice_end_call_output_get_call_id (
    QmiMessageVoiceEndCallOutput *self,
    guint8 *value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call ID' was not found in the message");
        return FALSE;
    }

    if (value_call_id)
        *value_call_id = self->arg_call_id;

    return TRUE;
}

gboolean
qmi_message_voice_end_call_output_get_result (
    QmiMessageVoiceEndCallOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_voice_end_call_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceEndCallOutput"),
                                          (GBoxedCopyFunc) qmi_message_voice_end_call_output_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_end_call_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceEndCallOutput *
qmi_message_voice_end_call_output_ref (QmiMessageVoiceEndCallOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_end_call_output_unref (QmiMessageVoiceEndCallOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceEndCallOutput, self);
    }
}

static gchar *
qmi_message_voice_end_call_input_call_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_END_CALL_INPUT_TLV_CALL_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_end_call_output_call_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_end_call_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_end_call_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_end_call_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_VOICE_END_CALL_INPUT_TLV_CALL_ID:
            tlv_type_str = "Call ID";
            translated_value = qmi_message_voice_end_call_input_call_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_CALL_ID:
            tlv_type_str = "Call ID";
            translated_value = qmi_message_voice_end_call_output_call_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_end_call_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"End Call\" (0x0021)\n",
                            line_prefix);

    {
        struct message_end_call_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_end_call_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageVoiceEndCallOutput *
__qmi_message_voice_end_call_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageVoiceEndCallOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_MESSAGE_VOICE_END_CALL);

    self = g_slice_new0 (QmiMessageVoiceEndCallOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0) {
                goto qmi_message_voice_end_call_output_call_id_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_call_id), NULL))
                goto qmi_message_voice_end_call_output_call_id_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call ID' TLV", offset);
            }

            self->arg_call_id_set = TRUE;

qmi_message_voice_end_call_output_call_id_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_END_CALL_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_voice_end_call_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_voice_end_call_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message VOICE Answer Call */


/* --- Input -- */

struct _QmiMessageVoiceAnswerCallInput {
    volatile gint ref_count;

    /* Call ID */
    gboolean arg_call_id_set;
    guint8 arg_call_id;
};

#define QMI_MESSAGE_VOICE_ANSWER_CALL_INPUT_TLV_CALL_ID 0x01

gboolean
qmi_message_voice_answer_call_input_get_call_id (
    QmiMessageVoiceAnswerCallInput *self,
    guint8 *value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call ID' was not found in the message");
        return FALSE;
    }

    if (value_call_id)
        *value_call_id = self->arg_call_id;

    return TRUE;
}

gboolean
qmi_message_voice_answer_call_input_set_call_id (
    QmiMessageVoiceAnswerCallInput *self,
    guint8 value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_call_id = value_call_id;
    self->arg_call_id_set = TRUE;

    return TRUE;
}

GType
qmi_message_voice_answer_call_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceAnswerCallInput"),
                                          (GBoxedCopyFunc) qmi_message_voice_answer_call_input_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_answer_call_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceAnswerCallInput *
qmi_message_voice_answer_call_input_ref (QmiMessageVoiceAnswerCallInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_answer_call_input_unref (QmiMessageVoiceAnswerCallInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceAnswerCallInput, self);
    }
}

QmiMessageVoiceAnswerCallInput *
qmi_message_voice_answer_call_input_new (void)
{
    QmiMessageVoiceAnswerCallInput *self;

    self = g_slice_new0 (QmiMessageVoiceAnswerCallInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_voice_answer_call_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageVoiceAnswerCallInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_VOICE,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_VOICE_ANSWER_CALL);

    /* There is at least one mandatory TLV, don't allow NULL input */
    if (!input) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Message 'Answer Call' has mandatory TLVs");
        goto error_out;
    }

    /* Try to add the 'Call ID' TLV */
    if (input->arg_call_id_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_ANSWER_CALL_INPUT_TLV_CALL_ID, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Call ID': ");
            goto error_out;
        }

        /* Write the guint8 variable to the buffer */
        if (!qmi_message_tlv_write_guint8 (self, input->arg_call_id, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Call ID': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Call ID': ");
            goto error_out;
        }
    } else {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_ARGS,
                     "Missing mandatory TLV 'Call ID' in message 'Answer Call'");
        goto error_out;
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageVoiceAnswerCallOutput {
    volatile gint ref_count;

    /* Call ID */
    gboolean arg_call_id_set;
    guint8 arg_call_id;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_CALL_ID 0x10
#define QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_RESULT 0x02

gboolean
qmi_message_voice_answer_call_output_get_call_id (
    QmiMessageVoiceAnswerCallOutput *self,
    guint8 *value_call_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call ID' was not found in the message");
        return FALSE;
    }

    if (value_call_id)
        *value_call_id = self->arg_call_id;

    return TRUE;
}

gboolean
qmi_message_voice_answer_call_output_get_result (
    QmiMessageVoiceAnswerCallOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_voice_answer_call_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceAnswerCallOutput"),
                                          (GBoxedCopyFunc) qmi_message_voice_answer_call_output_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_answer_call_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceAnswerCallOutput *
qmi_message_voice_answer_call_output_ref (QmiMessageVoiceAnswerCallOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_answer_call_output_unref (QmiMessageVoiceAnswerCallOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceAnswerCallOutput, self);
    }
}

static gchar *
qmi_message_voice_answer_call_input_call_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_ANSWER_CALL_INPUT_TLV_CALL_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_answer_call_output_call_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_answer_call_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_answer_call_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_answer_call_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_VOICE_ANSWER_CALL_INPUT_TLV_CALL_ID:
            tlv_type_str = "Call ID";
            translated_value = qmi_message_voice_answer_call_input_call_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_CALL_ID:
            tlv_type_str = "Call ID";
            translated_value = qmi_message_voice_answer_call_output_call_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_answer_call_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Answer Call\" (0x0022)\n",
                            line_prefix);

    {
        struct message_answer_call_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_answer_call_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageVoiceAnswerCallOutput *
__qmi_message_voice_answer_call_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageVoiceAnswerCallOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_MESSAGE_VOICE_ANSWER_CALL);

    self = g_slice_new0 (QmiMessageVoiceAnswerCallOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_CALL_ID, NULL, NULL)) == 0) {
                goto qmi_message_voice_answer_call_output_call_id_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_call_id), NULL))
                goto qmi_message_voice_answer_call_output_call_id_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call ID' TLV", offset);
            }

            self->arg_call_id_set = TRUE;

qmi_message_voice_answer_call_output_call_id_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_ANSWER_CALL_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_voice_answer_call_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_voice_answer_call_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* INDICATION: Qmi Indication VOICE All Call Status */


/* --- Output -- */

static void
remote_party_number_qmiindicationvoiceallcallstatusoutputremotepartynumbercall_clear (QmiIndicationVoiceAllCallStatusOutputRemotePartyNumberCall *p)
{
    g_free ((*p).type);
}

struct _QmiIndicationVoiceAllCallStatusOutput {
    volatile gint ref_count;

    /* Remote Party Number */
    gboolean arg_remote_party_number_set;
    GArray *arg_remote_party_number;

    /* Call Information */
    gboolean arg_call_information_set;
    GArray *arg_call_information;
};

#define QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_REMOTE_PARTY_NUMBER 0x10
#define QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_CALL_INFORMATION 0x01

gboolean
qmi_indication_voice_all_call_status_output_get_remote_party_number (
    QmiIndicationVoiceAllCallStatusOutput *self,
    GArray **value_remote_party_number,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_remote_party_number_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Remote Party Number' was not found in the message");
        return FALSE;
    }

    if (value_remote_party_number)
        *value_remote_party_number = self->arg_remote_party_number;

    return TRUE;
}

gboolean
qmi_indication_voice_all_call_status_output_get_call_information (
    QmiIndicationVoiceAllCallStatusOutput *self,
    GArray **value_call_information,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_information_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call Information' was not found in the message");
        return FALSE;
    }

    if (value_call_information)
        *value_call_information = self->arg_call_information;

    return TRUE;
}

GType
qmi_indication_voice_all_call_status_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiIndicationVoiceAllCallStatusOutput"),
                                          (GBoxedCopyFunc) qmi_indication_voice_all_call_status_output_ref,
                                          (GBoxedFreeFunc) qmi_indication_voice_all_call_status_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiIndicationVoiceAllCallStatusOutput *
qmi_indication_voice_all_call_status_output_ref (QmiIndicationVoiceAllCallStatusOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_indication_voice_all_call_status_output_unref (QmiIndicationVoiceAllCallStatusOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_remote_party_number)
            g_array_unref (self->arg_remote_party_number);
        if (self->arg_call_information)
            g_array_unref (self->arg_call_information);
        g_slice_free (QmiIndicationVoiceAllCallStatusOutput, self);
    }
}

static gchar *
qmi_indication_voice_all_call_status_output_remote_party_number_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_REMOTE_PARTY_NUMBER, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint remote_party_number_i;
        guint8 remote_party_number_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(remote_party_number_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (remote_party_number_i = 0; remote_party_number_i < remote_party_number_n_items; remote_party_number_i++) {
            g_string_append_printf (printable, " [%u] = '", remote_party_number_i);
            g_string_append (printable, "[");
            g_string_append (printable, " id = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%u", (guint)tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " presentation_indicator = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_PRESENTATION_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_presentation_get_string ((QmiVoicePresentation)tmp));
#elif defined  __QMI_VOICE_PRESENTATION_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_presentation_build_string_from_mask ((QmiVoicePresentation)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoicePresentation
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " type = '");

            {
                gchar *tmp;

                if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &tmp, &error))
                    goto out;
                g_string_append (printable, tmp);
                g_free (tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ]");
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_indication_voice_all_call_status_output_call_information_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_CALL_INFORMATION, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint call_information_i;
        guint8 call_information_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(call_information_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (call_information_i = 0; call_information_i < call_information_n_items; call_information_i++) {
            g_string_append_printf (printable, " [%u] = '", call_information_i);
            g_string_append (printable, "[");
            g_string_append (printable, " id = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%u", (guint)tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " state = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_CALL_STATE_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_call_state_get_string ((QmiVoiceCallState)tmp));
#elif defined  __QMI_VOICE_CALL_STATE_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_call_state_build_string_from_mask ((QmiVoiceCallState)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoiceCallState
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " type = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_CALL_TYPE_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_call_type_get_string ((QmiVoiceCallType)tmp));
#elif defined  __QMI_VOICE_CALL_TYPE_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_call_type_build_string_from_mask ((QmiVoiceCallType)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoiceCallType
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " direction = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_CALL_DIRECTION_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_call_direction_get_string ((QmiVoiceCallDirection)tmp));
#elif defined  __QMI_VOICE_CALL_DIRECTION_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_call_direction_build_string_from_mask ((QmiVoiceCallDirection)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoiceCallDirection
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " mode = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_CALL_MODE_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_call_mode_get_string ((QmiVoiceCallMode)tmp));
#elif defined  __QMI_VOICE_CALL_MODE_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_call_mode_build_string_from_mask ((QmiVoiceCallMode)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoiceCallMode
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " multipart_indicator = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
            }
            g_string_append (printable, "'");
            g_string_append (printable, " als = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_VOICE_ALS_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_voice_als_get_string ((QmiVoiceAls)tmp));
#elif defined  __QMI_VOICE_ALS_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_voice_als_build_string_from_mask ((QmiVoiceAls)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiVoiceAls
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ]");
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct indication_all_call_status_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
indication_all_call_status_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct indication_all_call_status_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    {
        switch (type) {
        case QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_REMOTE_PARTY_NUMBER:
            tlv_type_str = "Remote Party Number";
            translated_value = qmi_indication_voice_all_call_status_output_remote_party_number_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_CALL_INFORMATION:
            tlv_type_str = "Call Information";
            translated_value = qmi_indication_voice_all_call_status_output_call_information_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
indication_all_call_status_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"All Call Status\" (0x002E)\n",
                            line_prefix);

    {
        struct indication_all_call_status_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)indication_all_call_status_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiIndicationVoiceAllCallStatusOutput *
__qmi_indication_voice_all_call_status_indication_parse (
    QmiMessage *message,
    GError **error)
{
    QmiIndicationVoiceAllCallStatusOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_INDICATION_VOICE_ALL_CALL_STATUS);

    self = g_slice_new0 (QmiIndicationVoiceAllCallStatusOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_REMOTE_PARTY_NUMBER, NULL, NULL)) == 0) {
                goto qmi_indication_voice_all_call_status_output_remote_party_number_out;
            }
            {
                guint remote_party_number_i;
                guint8 remote_party_number_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(remote_party_number_n_items), NULL))
                    goto qmi_indication_voice_all_call_status_output_remote_party_number_out;

                self->arg_remote_party_number = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (QmiIndicationVoiceAllCallStatusOutputRemotePartyNumberCall),
                    (guint)remote_party_number_n_items);

                g_array_set_clear_func (self->arg_remote_party_number,
                                        (GDestroyNotify)remote_party_number_qmiindicationvoiceallcallstatusoutputremotepartynumbercall_clear);

                for (remote_party_number_i = 0; remote_party_number_i < remote_party_number_n_items; remote_party_number_i++) {
                    QmiIndicationVoiceAllCallStatusOutputRemotePartyNumberCall remote_party_number_aux;

                    if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(remote_party_number_aux.id), NULL))
                        goto qmi_indication_voice_all_call_status_output_remote_party_number_out;
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                            goto qmi_indication_voice_all_call_status_output_remote_party_number_out;
                        remote_party_number_aux.presentation_indicator = (QmiVoicePresentation)tmp;
                    }
                    if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &(remote_party_number_aux.type), NULL))
                        goto qmi_indication_voice_all_call_status_output_remote_party_number_out;
                    g_array_insert_val (self->arg_remote_party_number, remote_party_number_i, remote_party_number_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Remote Party Number' TLV", offset);
            }

            self->arg_remote_party_number_set = TRUE;

qmi_indication_voice_all_call_status_output_remote_party_number_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT_TLV_CALL_INFORMATION, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Call Information TLV: ");
                qmi_indication_voice_all_call_status_output_unref (self);
                return NULL;
            }
            {
                guint call_information_i;
                guint8 call_information_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(call_information_n_items), error))
                    goto qmi_indication_voice_all_call_status_output_call_information_out;

                self->arg_call_information = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (QmiIndicationVoiceAllCallStatusOutputCallInformationCall),
                    (guint)call_information_n_items);

                for (call_information_i = 0; call_information_i < call_information_n_items; call_information_i++) {
                    QmiIndicationVoiceAllCallStatusOutputCallInformationCall call_information_aux;

                    if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(call_information_aux.id), error))
                        goto qmi_indication_voice_all_call_status_output_call_information_out;
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.state = (QmiVoiceCallState)tmp;
                    }
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.type = (QmiVoiceCallType)tmp;
                    }
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.direction = (QmiVoiceCallDirection)tmp;
                    }
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.mode = (QmiVoiceCallMode)tmp;
                    }
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.multipart_indicator = (gboolean)tmp;
                    }
                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_indication_voice_all_call_status_output_call_information_out;
                        call_information_aux.als = (QmiVoiceAls)tmp;
                    }
                    g_array_insert_val (self->arg_call_information, call_information_i, call_information_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call Information' TLV", offset);
            }

            self->arg_call_information_set = TRUE;

qmi_indication_voice_all_call_status_output_call_information_out:
            if (!self->arg_call_information_set) {
                qmi_indication_voice_all_call_status_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message VOICE Get Config */


/* --- Input -- */

struct _QmiMessageVoiceGetConfigInput {
    volatile gint ref_count;

    /* Voice Domain Preference */
    gboolean arg_voice_domain_preference_set;
    guint8 arg_voice_domain_preference;

    /* NAM Index */
    gboolean arg_nam_index_set;
    guint8 arg_nam_index;

    /* Preferred Voice Privacy */
    gboolean arg_preferred_voice_privacy_set;
    guint8 arg_preferred_voice_privacy;

    /* AMR Status */
    gboolean arg_amr_status_set;
    guint8 arg_amr_status;

    /* Preferred Voice Service Option */
    gboolean arg_preferred_voice_service_option_set;
    guint8 arg_preferred_voice_service_option;

    /* TTY Mode */
    gboolean arg_tty_mode_set;
    guint8 arg_tty_mode;

    /* Roam Timer */
    gboolean arg_roam_timer_set;
    guint8 arg_roam_timer;

    /* Air Timer */
    gboolean arg_air_timer_set;
    guint8 arg_air_timer;

    /* Auto Answer */
    gboolean arg_auto_answer_set;
    guint8 arg_auto_answer;
};

#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_VOICE_DOMAIN_PREFERENCE 0x18
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_NAM_INDEX 0x17
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_PRIVACY 0x16
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AMR_STATUS 0x15
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_SERVICE_OPTION 0x14
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_TTY_MODE 0x13
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_ROAM_TIMER 0x12
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AIR_TIMER 0x11
#define QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AUTO_ANSWER 0x10

gboolean
qmi_message_voice_get_config_input_get_voice_domain_preference (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_voice_domain_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_voice_domain_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Voice Domain Preference' was not found in the message");
        return FALSE;
    }

    if (value_voice_domain_preference)
        *value_voice_domain_preference = (gboolean)(self->arg_voice_domain_preference);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_voice_domain_preference (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_voice_domain_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_voice_domain_preference = (guint8)(value_voice_domain_preference);
    self->arg_voice_domain_preference_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_nam_index (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_nam_index,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_nam_index_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'NAM Index' was not found in the message");
        return FALSE;
    }

    if (value_nam_index)
        *value_nam_index = (gboolean)(self->arg_nam_index);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_nam_index (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_nam_index,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_nam_index = (guint8)(value_nam_index);
    self->arg_nam_index_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_preferred_voice_privacy (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_preferred_voice_privacy,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_preferred_voice_privacy_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Preferred Voice Privacy' was not found in the message");
        return FALSE;
    }

    if (value_preferred_voice_privacy)
        *value_preferred_voice_privacy = (gboolean)(self->arg_preferred_voice_privacy);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_preferred_voice_privacy (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_preferred_voice_privacy,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_preferred_voice_privacy = (guint8)(value_preferred_voice_privacy);
    self->arg_preferred_voice_privacy_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_amr_status (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_amr_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_amr_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'AMR Status' was not found in the message");
        return FALSE;
    }

    if (value_amr_status)
        *value_amr_status = (gboolean)(self->arg_amr_status);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_amr_status (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_amr_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_amr_status = (guint8)(value_amr_status);
    self->arg_amr_status_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_preferred_voice_service_option (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_preferred_voice_service_option,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_preferred_voice_service_option_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Preferred Voice Service Option' was not found in the message");
        return FALSE;
    }

    if (value_preferred_voice_service_option)
        *value_preferred_voice_service_option = (gboolean)(self->arg_preferred_voice_service_option);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_preferred_voice_service_option (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_preferred_voice_service_option,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_preferred_voice_service_option = (guint8)(value_preferred_voice_service_option);
    self->arg_preferred_voice_service_option_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_tty_mode (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_tty_mode,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tty_mode_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'TTY Mode' was not found in the message");
        return FALSE;
    }

    if (value_tty_mode)
        *value_tty_mode = (gboolean)(self->arg_tty_mode);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_tty_mode (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_tty_mode,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_tty_mode = (guint8)(value_tty_mode);
    self->arg_tty_mode_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_roam_timer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_roam_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_roam_timer_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Roam Timer' was not found in the message");
        return FALSE;
    }

    if (value_roam_timer)
        *value_roam_timer = (gboolean)(self->arg_roam_timer);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_roam_timer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_roam_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_roam_timer = (guint8)(value_roam_timer);
    self->arg_roam_timer_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_air_timer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_air_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_air_timer_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Air Timer' was not found in the message");
        return FALSE;
    }

    if (value_air_timer)
        *value_air_timer = (gboolean)(self->arg_air_timer);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_air_timer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_air_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_air_timer = (guint8)(value_air_timer);
    self->arg_air_timer_set = TRUE;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_get_auto_answer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean *value_auto_answer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_auto_answer_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Auto Answer' was not found in the message");
        return FALSE;
    }

    if (value_auto_answer)
        *value_auto_answer = (gboolean)(self->arg_auto_answer);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_input_set_auto_answer (
    QmiMessageVoiceGetConfigInput *self,
    gboolean value_auto_answer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_auto_answer = (guint8)(value_auto_answer);
    self->arg_auto_answer_set = TRUE;

    return TRUE;
}

GType
qmi_message_voice_get_config_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceGetConfigInput"),
                                          (GBoxedCopyFunc) qmi_message_voice_get_config_input_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_get_config_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceGetConfigInput *
qmi_message_voice_get_config_input_ref (QmiMessageVoiceGetConfigInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_get_config_input_unref (QmiMessageVoiceGetConfigInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceGetConfigInput, self);
    }
}

QmiMessageVoiceGetConfigInput *
qmi_message_voice_get_config_input_new (void)
{
    QmiMessageVoiceGetConfigInput *self;

    self = g_slice_new0 (QmiMessageVoiceGetConfigInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_voice_get_config_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageVoiceGetConfigInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_VOICE,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_VOICE_GET_CONFIG);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Voice Domain Preference' TLV */
    if (input->arg_voice_domain_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_VOICE_DOMAIN_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Voice Domain Preference': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_voice_domain_preference;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Voice Domain Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Voice Domain Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'NAM Index' TLV */
    if (input->arg_nam_index_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_NAM_INDEX, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'NAM Index': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_nam_index;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'NAM Index': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'NAM Index': ");
            goto error_out;
        }
    }

    /* Try to add the 'Preferred Voice Privacy' TLV */
    if (input->arg_preferred_voice_privacy_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_PRIVACY, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Preferred Voice Privacy': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_preferred_voice_privacy;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Preferred Voice Privacy': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Preferred Voice Privacy': ");
            goto error_out;
        }
    }

    /* Try to add the 'AMR Status' TLV */
    if (input->arg_amr_status_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AMR_STATUS, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'AMR Status': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_amr_status;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'AMR Status': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'AMR Status': ");
            goto error_out;
        }
    }

    /* Try to add the 'Preferred Voice Service Option' TLV */
    if (input->arg_preferred_voice_service_option_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_SERVICE_OPTION, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Preferred Voice Service Option': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_preferred_voice_service_option;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Preferred Voice Service Option': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Preferred Voice Service Option': ");
            goto error_out;
        }
    }

    /* Try to add the 'TTY Mode' TLV */
    if (input->arg_tty_mode_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_TTY_MODE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'TTY Mode': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_tty_mode;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'TTY Mode': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'TTY Mode': ");
            goto error_out;
        }
    }

    /* Try to add the 'Roam Timer' TLV */
    if (input->arg_roam_timer_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_ROAM_TIMER, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Roam Timer': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_roam_timer;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Roam Timer': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Roam Timer': ");
            goto error_out;
        }
    }

    /* Try to add the 'Air Timer' TLV */
    if (input->arg_air_timer_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AIR_TIMER, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Air Timer': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_air_timer;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Air Timer': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Air Timer': ");
            goto error_out;
        }
    }

    /* Try to add the 'Auto Answer' TLV */
    if (input->arg_auto_answer_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AUTO_ANSWER, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Auto Answer': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_auto_answer;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Auto Answer': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Auto Answer': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageVoiceGetConfigOutput {
    volatile gint ref_count;

    /* Current Voice Domain Preference */
    gboolean arg_current_voice_domain_preference_set;
    guint8 arg_current_voice_domain_preference;

    /* Current Voice Privacy Preference */
    gboolean arg_current_voice_privacy_preference_set;
    guint8 arg_current_voice_privacy_preference;

    /* Current AMR Status */
    gboolean arg_current_amr_status_set;
    guint8 arg_current_amr_status_gsm;
    guint8 arg_current_amr_status_wcdma;

    /* Current Preferred Voice SO */
    gboolean arg_current_preferred_voice_so_set;
    guint8 arg_current_preferred_voice_so_nam_id;
    guint8 arg_current_preferred_voice_so_evrc_capability;
    guint16 arg_current_preferred_voice_so_home_page_voice_service_option;
    guint16 arg_current_preferred_voice_so_home_origination_voice_service_option;
    guint16 arg_current_preferred_voice_so_roaming_origination_voice_service_option;

    /* Current TTY Mode */
    gboolean arg_current_tty_mode_set;
    guint8 arg_current_tty_mode;

    /* Roam Timer Count */
    gboolean arg_roam_timer_count_set;
    guint8 arg_roam_timer_count_nam_id;
    guint32 arg_roam_timer_count_roam_timer;

    /* Air Timer Count */
    gboolean arg_air_timer_count_set;
    guint8 arg_air_timer_count_nam_id;
    guint32 arg_air_timer_count_air_timer;

    /* Auto Answer Status */
    gboolean arg_auto_answer_status_set;
    guint8 arg_auto_answer_status;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_DOMAIN_PREFERENCE 0x17
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_PRIVACY_PREFERENCE 0x16
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_AMR_STATUS 0x15
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_PREFERRED_VOICE_SO 0x14
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_TTY_MODE 0x13
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_ROAM_TIMER_COUNT 0x12
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AIR_TIMER_COUNT 0x11
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AUTO_ANSWER_STATUS 0x10
#define QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_RESULT 0x02

gboolean
qmi_message_voice_get_config_output_get_current_voice_domain_preference (
    QmiMessageVoiceGetConfigOutput *self,
    QmiVoiceDomain *value_current_voice_domain_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_voice_domain_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current Voice Domain Preference' was not found in the message");
        return FALSE;
    }

    if (value_current_voice_domain_preference)
        *value_current_voice_domain_preference = (QmiVoiceDomain)(self->arg_current_voice_domain_preference);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_current_voice_privacy_preference (
    QmiMessageVoiceGetConfigOutput *self,
    QmiVoicePrivacy *value_current_voice_privacy_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_voice_privacy_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current Voice Privacy Preference' was not found in the message");
        return FALSE;
    }

    if (value_current_voice_privacy_preference)
        *value_current_voice_privacy_preference = (QmiVoicePrivacy)(self->arg_current_voice_privacy_preference);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_current_amr_status (
    QmiMessageVoiceGetConfigOutput *self,
    gboolean *value_current_amr_status_gsm,
    QmiVoiceWcdmaAmrStatus *value_current_amr_status_wcdma,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_amr_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current AMR Status' was not found in the message");
        return FALSE;
    }

    if (value_current_amr_status_gsm)
        *value_current_amr_status_gsm = (gboolean)(self->arg_current_amr_status_gsm);
    if (value_current_amr_status_wcdma)
        *value_current_amr_status_wcdma = (QmiVoiceWcdmaAmrStatus)(self->arg_current_amr_status_wcdma);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_current_preferred_voice_so (
    QmiMessageVoiceGetConfigOutput *self,
    guint8 *value_current_preferred_voice_so_nam_id,
    gboolean *value_current_preferred_voice_so_evrc_capability,
    QmiVoiceServiceOption *value_current_preferred_voice_so_home_page_voice_service_option,
    QmiVoiceServiceOption *value_current_preferred_voice_so_home_origination_voice_service_option,
    QmiVoiceServiceOption *value_current_preferred_voice_so_roaming_origination_voice_service_option,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_preferred_voice_so_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current Preferred Voice SO' was not found in the message");
        return FALSE;
    }

    if (value_current_preferred_voice_so_nam_id)
        *value_current_preferred_voice_so_nam_id = self->arg_current_preferred_voice_so_nam_id;
    if (value_current_preferred_voice_so_evrc_capability)
        *value_current_preferred_voice_so_evrc_capability = (gboolean)(self->arg_current_preferred_voice_so_evrc_capability);
    if (value_current_preferred_voice_so_home_page_voice_service_option)
        *value_current_preferred_voice_so_home_page_voice_service_option = (QmiVoiceServiceOption)(self->arg_current_preferred_voice_so_home_page_voice_service_option);
    if (value_current_preferred_voice_so_home_origination_voice_service_option)
        *value_current_preferred_voice_so_home_origination_voice_service_option = (QmiVoiceServiceOption)(self->arg_current_preferred_voice_so_home_origination_voice_service_option);
    if (value_current_preferred_voice_so_roaming_origination_voice_service_option)
        *value_current_preferred_voice_so_roaming_origination_voice_service_option = (QmiVoiceServiceOption)(self->arg_current_preferred_voice_so_roaming_origination_voice_service_option);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_current_tty_mode (
    QmiMessageVoiceGetConfigOutput *self,
    QmiVoiceTtyMode *value_current_tty_mode,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_tty_mode_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current TTY Mode' was not found in the message");
        return FALSE;
    }

    if (value_current_tty_mode)
        *value_current_tty_mode = (QmiVoiceTtyMode)(self->arg_current_tty_mode);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_roam_timer_count (
    QmiMessageVoiceGetConfigOutput *self,
    guint8 *value_roam_timer_count_nam_id,
    guint32 *value_roam_timer_count_roam_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_roam_timer_count_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Roam Timer Count' was not found in the message");
        return FALSE;
    }

    if (value_roam_timer_count_nam_id)
        *value_roam_timer_count_nam_id = self->arg_roam_timer_count_nam_id;
    if (value_roam_timer_count_roam_timer)
        *value_roam_timer_count_roam_timer = self->arg_roam_timer_count_roam_timer;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_air_timer_count (
    QmiMessageVoiceGetConfigOutput *self,
    guint8 *value_air_timer_count_nam_id,
    guint32 *value_air_timer_count_air_timer,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_air_timer_count_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Air Timer Count' was not found in the message");
        return FALSE;
    }

    if (value_air_timer_count_nam_id)
        *value_air_timer_count_nam_id = self->arg_air_timer_count_nam_id;
    if (value_air_timer_count_air_timer)
        *value_air_timer_count_air_timer = self->arg_air_timer_count_air_timer;

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_auto_answer_status (
    QmiMessageVoiceGetConfigOutput *self,
    gboolean *value_auto_answer_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_auto_answer_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Auto Answer Status' was not found in the message");
        return FALSE;
    }

    if (value_auto_answer_status)
        *value_auto_answer_status = (gboolean)(self->arg_auto_answer_status);

    return TRUE;
}

gboolean
qmi_message_voice_get_config_output_get_result (
    QmiMessageVoiceGetConfigOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_voice_get_config_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageVoiceGetConfigOutput"),
                                          (GBoxedCopyFunc) qmi_message_voice_get_config_output_ref,
                                          (GBoxedFreeFunc) qmi_message_voice_get_config_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

QmiMessageVoiceGetConfigOutput *
qmi_message_voice_get_config_output_ref (QmiMessageVoiceGetConfigOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

void
qmi_message_voice_get_config_output_unref (QmiMessageVoiceGetConfigOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageVoiceGetConfigOutput, self);
    }
}

static gchar *
qmi_message_voice_get_config_input_voice_domain_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_VOICE_DOMAIN_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_nam_index_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_NAM_INDEX, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_preferred_voice_privacy_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_PRIVACY, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_amr_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AMR_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_preferred_voice_service_option_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_SERVICE_OPTION, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_tty_mode_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_TTY_MODE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_roam_timer_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_ROAM_TIMER, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_air_timer_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AIR_TIMER, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_input_auto_answer_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AUTO_ANSWER, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_current_voice_domain_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_DOMAIN_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_DOMAIN_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_domain_get_string ((QmiVoiceDomain)tmp));
#elif defined  __QMI_VOICE_DOMAIN_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_domain_build_string_from_mask ((QmiVoiceDomain)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceDomain
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_current_voice_privacy_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_PRIVACY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_PRIVACY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_privacy_get_string ((QmiVoicePrivacy)tmp));
#elif defined  __QMI_VOICE_PRIVACY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_privacy_build_string_from_mask ((QmiVoicePrivacy)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoicePrivacy
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_current_amr_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_AMR_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " gsm = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }
    g_string_append (printable, "'");
    g_string_append (printable, " wcdma = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_WCDMA_AMR_STATUS_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_wcdma_amr_status_get_string ((QmiVoiceWcdmaAmrStatus)tmp));
#elif defined  __QMI_VOICE_WCDMA_AMR_STATUS_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_wcdma_amr_status_build_string_from_mask ((QmiVoiceWcdmaAmrStatus)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceWcdmaAmrStatus
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_current_preferred_voice_so_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_PREFERRED_VOICE_SO, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " nam_id = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " evrc_capability = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }
    g_string_append (printable, "'");
    g_string_append (printable, " home_page_voice_service_option = '");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_SERVICE_OPTION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_service_option_get_string ((QmiVoiceServiceOption)tmp));
#elif defined  __QMI_VOICE_SERVICE_OPTION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_service_option_build_string_from_mask ((QmiVoiceServiceOption)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceServiceOption
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " home_origination_voice_service_option = '");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_SERVICE_OPTION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_service_option_get_string ((QmiVoiceServiceOption)tmp));
#elif defined  __QMI_VOICE_SERVICE_OPTION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_service_option_build_string_from_mask ((QmiVoiceServiceOption)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceServiceOption
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " roaming_origination_voice_service_option = '");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_SERVICE_OPTION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_service_option_get_string ((QmiVoiceServiceOption)tmp));
#elif defined  __QMI_VOICE_SERVICE_OPTION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_service_option_build_string_from_mask ((QmiVoiceServiceOption)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceServiceOption
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_current_tty_mode_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_TTY_MODE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_VOICE_TTY_MODE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_voice_tty_mode_get_string ((QmiVoiceTtyMode)tmp));
#elif defined  __QMI_VOICE_TTY_MODE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_voice_tty_mode_build_string_from_mask ((QmiVoiceTtyMode)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiVoiceTtyMode
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_roam_timer_count_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_ROAM_TIMER_COUNT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " nam_id = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " roam_timer = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_air_timer_count_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AIR_TIMER_COUNT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " nam_id = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " air_timer = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_voice_get_config_output_auto_answer_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AUTO_ANSWER_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_config_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_config_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_config_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_VOICE_DOMAIN_PREFERENCE:
            tlv_type_str = "Voice Domain Preference";
            translated_value = qmi_message_voice_get_config_input_voice_domain_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_NAM_INDEX:
            tlv_type_str = "NAM Index";
            translated_value = qmi_message_voice_get_config_input_nam_index_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_PRIVACY:
            tlv_type_str = "Preferred Voice Privacy";
            translated_value = qmi_message_voice_get_config_input_preferred_voice_privacy_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AMR_STATUS:
            tlv_type_str = "AMR Status";
            translated_value = qmi_message_voice_get_config_input_amr_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_PREFERRED_VOICE_SERVICE_OPTION:
            tlv_type_str = "Preferred Voice Service Option";
            translated_value = qmi_message_voice_get_config_input_preferred_voice_service_option_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_TTY_MODE:
            tlv_type_str = "TTY Mode";
            translated_value = qmi_message_voice_get_config_input_tty_mode_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_ROAM_TIMER:
            tlv_type_str = "Roam Timer";
            translated_value = qmi_message_voice_get_config_input_roam_timer_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AIR_TIMER:
            tlv_type_str = "Air Timer";
            translated_value = qmi_message_voice_get_config_input_air_timer_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_INPUT_TLV_AUTO_ANSWER:
            tlv_type_str = "Auto Answer";
            translated_value = qmi_message_voice_get_config_input_auto_answer_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_DOMAIN_PREFERENCE:
            tlv_type_str = "Current Voice Domain Preference";
            translated_value = qmi_message_voice_get_config_output_current_voice_domain_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_PRIVACY_PREFERENCE:
            tlv_type_str = "Current Voice Privacy Preference";
            translated_value = qmi_message_voice_get_config_output_current_voice_privacy_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_AMR_STATUS:
            tlv_type_str = "Current AMR Status";
            translated_value = qmi_message_voice_get_config_output_current_amr_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_PREFERRED_VOICE_SO:
            tlv_type_str = "Current Preferred Voice SO";
            translated_value = qmi_message_voice_get_config_output_current_preferred_voice_so_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_TTY_MODE:
            tlv_type_str = "Current TTY Mode";
            translated_value = qmi_message_voice_get_config_output_current_tty_mode_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_ROAM_TIMER_COUNT:
            tlv_type_str = "Roam Timer Count";
            translated_value = qmi_message_voice_get_config_output_roam_timer_count_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AIR_TIMER_COUNT:
            tlv_type_str = "Air Timer Count";
            translated_value = qmi_message_voice_get_config_output_air_timer_count_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AUTO_ANSWER_STATUS:
            tlv_type_str = "Auto Answer Status";
            translated_value = qmi_message_voice_get_config_output_auto_answer_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_config_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Config\" (0x0041)\n",
                            line_prefix);

    {
        struct message_get_config_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_config_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageVoiceGetConfigOutput *
__qmi_message_voice_get_config_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageVoiceGetConfigOutput *self;

    g_assert_cmphex (qmi_message_get_message_id (message), ==, QMI_MESSAGE_VOICE_GET_CONFIG);

    self = g_slice_new0 (QmiMessageVoiceGetConfigOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_DOMAIN_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_current_voice_domain_preference_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_voice_domain_preference_out;
                self->arg_current_voice_domain_preference = (QmiVoiceDomain)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current Voice Domain Preference' TLV", offset);
            }

            self->arg_current_voice_domain_preference_set = TRUE;

qmi_message_voice_get_config_output_current_voice_domain_preference_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_VOICE_PRIVACY_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_current_voice_privacy_preference_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_voice_privacy_preference_out;
                self->arg_current_voice_privacy_preference = (QmiVoicePrivacy)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current Voice Privacy Preference' TLV", offset);
            }

            self->arg_current_voice_privacy_preference_set = TRUE;

qmi_message_voice_get_config_output_current_voice_privacy_preference_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_AMR_STATUS, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_current_amr_status_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_amr_status_out;
                self->arg_current_amr_status_gsm = (gboolean)tmp;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_amr_status_out;
                self->arg_current_amr_status_wcdma = (QmiVoiceWcdmaAmrStatus)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current AMR Status' TLV", offset);
            }

            self->arg_current_amr_status_set = TRUE;

qmi_message_voice_get_config_output_current_amr_status_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_PREFERRED_VOICE_SO, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_current_preferred_voice_so_nam_id), NULL))
                goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
                self->arg_current_preferred_voice_so_evrc_capability = (gboolean)tmp;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
                self->arg_current_preferred_voice_so_home_page_voice_service_option = (QmiVoiceServiceOption)tmp;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
                self->arg_current_preferred_voice_so_home_origination_voice_service_option = (QmiVoiceServiceOption)tmp;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_preferred_voice_so_out;
                self->arg_current_preferred_voice_so_roaming_origination_voice_service_option = (QmiVoiceServiceOption)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current Preferred Voice SO' TLV", offset);
            }

            self->arg_current_preferred_voice_so_set = TRUE;

qmi_message_voice_get_config_output_current_preferred_voice_so_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_CURRENT_TTY_MODE, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_current_tty_mode_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_current_tty_mode_out;
                self->arg_current_tty_mode = (QmiVoiceTtyMode)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current TTY Mode' TLV", offset);
            }

            self->arg_current_tty_mode_set = TRUE;

qmi_message_voice_get_config_output_current_tty_mode_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_ROAM_TIMER_COUNT, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_roam_timer_count_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_roam_timer_count_nam_id), NULL))
                goto qmi_message_voice_get_config_output_roam_timer_count_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_roam_timer_count_roam_timer), NULL))
                goto qmi_message_voice_get_config_output_roam_timer_count_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Roam Timer Count' TLV", offset);
            }

            self->arg_roam_timer_count_set = TRUE;

qmi_message_voice_get_config_output_roam_timer_count_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AIR_TIMER_COUNT, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_air_timer_count_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_air_timer_count_nam_id), NULL))
                goto qmi_message_voice_get_config_output_air_timer_count_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_air_timer_count_air_timer), NULL))
                goto qmi_message_voice_get_config_output_air_timer_count_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Air Timer Count' TLV", offset);
            }

            self->arg_air_timer_count_set = TRUE;

qmi_message_voice_get_config_output_air_timer_count_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_AUTO_ANSWER_STATUS, NULL, NULL)) == 0) {
                goto qmi_message_voice_get_config_output_auto_answer_status_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_voice_get_config_output_auto_answer_status_out;
                self->arg_auto_answer_status = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Auto Answer Status' TLV", offset);
            }

            self->arg_auto_answer_status_set = TRUE;

qmi_message_voice_get_config_output_auto_answer_status_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_VOICE_GET_CONFIG_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_voice_get_config_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_voice_get_config_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* Service-specific utils: VOICE */


gchar *
__qmi_message_voice_get_printable (
    QmiMessage *self,
    QmiMessageContext *context,
    const gchar *line_prefix)
{
    if (qmi_message_is_indication (self)) {
        switch (qmi_message_get_message_id (self)) {
        case QMI_INDICATION_VOICE_ALL_CALL_STATUS:
            return indication_all_call_status_get_printable (self, line_prefix);
        default:
             return NULL;
        }
    } else {
        guint16 vendor_id;

        vendor_id = (context ? qmi_message_context_get_vendor_id (context) : QMI_MESSAGE_VENDOR_GENERIC);
        if (vendor_id == QMI_MESSAGE_VENDOR_GENERIC) {
            switch (qmi_message_get_message_id (self)) {
            case QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES:
                return message_get_supported_messages_get_printable (self, line_prefix);
            case QMI_MESSAGE_VOICE_DIAL_CALL:
                return message_dial_call_get_printable (self, line_prefix);
            case QMI_MESSAGE_VOICE_END_CALL:
                return message_end_call_get_printable (self, line_prefix);
            case QMI_MESSAGE_VOICE_ANSWER_CALL:
                return message_answer_call_get_printable (self, line_prefix);
            case QMI_MESSAGE_VOICE_GET_CONFIG:
                return message_get_config_get_printable (self, line_prefix);
             default:
                 return NULL;
            }
        } else {
            return NULL;
        }
    }
}

gboolean
__qmi_message_voice_get_version_introduced (
    QmiMessage *self,
    QmiMessageContext *context,
    guint *major,
    guint *minor)
{
    guint16 vendor_id;

    vendor_id = (context ? qmi_message_context_get_vendor_id (context) : QMI_MESSAGE_VENDOR_GENERIC);
    if (vendor_id == QMI_MESSAGE_VENDOR_GENERIC) {
        switch (qmi_message_get_message_id (self)) {
        case QMI_MESSAGE_VOICE_GET_SUPPORTED_MESSAGES:
            *major = 2;
            *minor = 21;
            return TRUE;
        case QMI_MESSAGE_VOICE_DIAL_CALL:
            *major = 1;
            *minor = 0;
            return TRUE;
        case QMI_MESSAGE_VOICE_END_CALL:
            *major = 1;
            *minor = 0;
            return TRUE;
        case QMI_MESSAGE_VOICE_ANSWER_CALL:
            *major = 1;
            *minor = 0;
            return TRUE;
        case QMI_MESSAGE_VOICE_GET_CONFIG:
            *major = 2;
            *minor = 1;
            return TRUE;
        default:
            return FALSE;
        }
    } else {
        return FALSE;
    }
}

/*****************************************************************************/
/* CLIENT: QMI Client Voice */


/**
 * SECTION: qmi-client-voice
 * @title: QmiClientVoice
 * @short_description: #QmiClient for the VOICE service.
 *
 * #QmiClient which handles operations in the VOICE service.
 */

G_DEFINE_TYPE (QmiClientVoice, qmi_client_voice, QMI_TYPE_CLIENT);

enum {
    SIGNAL_ALL_CALL_STATUS,
    SIGNAL_LAST
};

static guint signals[SIGNAL_LAST] = { 0 };

static void
process_indication (QmiClient *self,
                    QmiMessage *message)
{
    switch (qmi_message_get_message_id (message)) {
        case QMI_INDICATION_VOICE_ALL_CALL_STATUS: {
            QmiIndicationVoiceAllCallStatusOutput *output;
            GError *error = NULL;

            /* Parse indication */
            output = __qmi_indication_voice_all_call_status_indication_parse (message, &error);
            if (!output) {
                g_warning ("Couldn't parse 'All Call Status' indication: %s",
                           error ? error->message : "Unknown error");
                if (error)
                    g_error_free (error);
            } else {
                g_signal_emit (self, signals[SIGNAL_ALL_CALL_STATUS], 0, output);
                qmi_indication_voice_all_call_status_output_unref (output);
            }
            break;
        }
        default:
            break;
    }
}

static void
qmi_client_voice_init (QmiClientVoice *self)
{
}

static void
qmi_client_voice_class_init (QmiClientVoiceClass *klass)
{
    QmiClientClass *client_class = QMI_CLIENT_CLASS (klass);

    client_class->process_indication = process_indication;

    /**
     * QmiClientVoice::all-call-status:
     * @object: A #QmiClientVoice.
     * @output: A #QmiIndicationVoiceAllCallStatusOutput.
     *
     * The ::all-call-status signal gets emitted when a '<link linkend="libqmi-glib-VOICE-All-Call-Status-indication.top_of_page">All Call Status</link>' indication is received.
     *
     * Since: 1.14
     */
    signals[SIGNAL_ALL_CALL_STATUS] =
        g_signal_new ("all-call-status",
                      G_OBJECT_CLASS_TYPE (G_OBJECT_CLASS (klass)),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      QMI_TYPE_INDICATION_VOICE_ALL_CALL_STATUS_OUTPUT);
}


QmiMessageVoiceGetSupportedMessagesOutput *
qmi_client_voice_get_supported_messages_finish (
    QmiClientVoice *self,
    GAsyncResult *res,
    GError **error)
{
   return g_task_propagate_pointer (G_TASK (res), error);
}

static void
get_supported_messages_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GTask *task)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageVoiceGetSupportedMessagesOutput *output;

    reply = qmi_device_command_full_finish (device, res, &error);
    if (!reply) {
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    /* Parse reply */
    output = __qmi_message_voice_get_supported_messages_response_parse (reply, &error);
    if (!output)
        g_task_return_error (task, error);
    else
        g_task_return_pointer (task,
                               output,
                               (GDestroyNotify)qmi_message_voice_get_supported_messages_output_unref);
    g_object_unref (task);
    qmi_message_unref (reply);
}

void
qmi_client_voice_get_supported_messages (
    QmiClientVoice *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GTask *task;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    task = g_task_new (self, cancellable, callback, user_data);
    if (!qmi_client_is_valid (QMI_CLIENT (self))) {
        g_task_return_new_error (task, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");
        g_object_unref (task);
        return;
    }

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_voice_get_supported_messages_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    qmi_device_command_full (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                             request,
                             NULL,
                             timeout,
                             cancellable,
                             (GAsyncReadyCallback)get_supported_messages_ready,
                             task);
    qmi_message_unref (request);
}


QmiMessageVoiceDialCallOutput *
qmi_client_voice_dial_call_finish (
    QmiClientVoice *self,
    GAsyncResult *res,
    GError **error)
{
   return g_task_propagate_pointer (G_TASK (res), error);
}

static void
dial_call_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GTask *task)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageVoiceDialCallOutput *output;

    reply = qmi_device_command_full_finish (device, res, &error);
    if (!reply) {
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    /* Parse reply */
    output = __qmi_message_voice_dial_call_response_parse (reply, &error);
    if (!output)
        g_task_return_error (task, error);
    else
        g_task_return_pointer (task,
                               output,
                               (GDestroyNotify)qmi_message_voice_dial_call_output_unref);
    g_object_unref (task);
    qmi_message_unref (reply);
}

void
qmi_client_voice_dial_call (
    QmiClientVoice *self,
    QmiMessageVoiceDialCallInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GTask *task;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    task = g_task_new (self, cancellable, callback, user_data);
    if (!qmi_client_is_valid (QMI_CLIENT (self))) {
        g_task_return_new_error (task, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");
        g_object_unref (task);
        return;
    }

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_voice_dial_call_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    qmi_device_command_full (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                             request,
                             NULL,
                             timeout,
                             cancellable,
                             (GAsyncReadyCallback)dial_call_ready,
                             task);
    qmi_message_unref (request);
}


QmiMessageVoiceEndCallOutput *
qmi_client_voice_end_call_finish (
    QmiClientVoice *self,
    GAsyncResult *res,
    GError **error)
{
   return g_task_propagate_pointer (G_TASK (res), error);
}

static void
end_call_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GTask *task)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageVoiceEndCallOutput *output;

    reply = qmi_device_command_full_finish (device, res, &error);
    if (!reply) {
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    /* Parse reply */
    output = __qmi_message_voice_end_call_response_parse (reply, &error);
    if (!output)
        g_task_return_error (task, error);
    else
        g_task_return_pointer (task,
                               output,
                               (GDestroyNotify)qmi_message_voice_end_call_output_unref);
    g_object_unref (task);
    qmi_message_unref (reply);
}

void
qmi_client_voice_end_call (
    QmiClientVoice *self,
    QmiMessageVoiceEndCallInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GTask *task;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    task = g_task_new (self, cancellable, callback, user_data);
    if (!qmi_client_is_valid (QMI_CLIENT (self))) {
        g_task_return_new_error (task, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");
        g_object_unref (task);
        return;
    }

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_voice_end_call_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    qmi_device_command_full (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                             request,
                             NULL,
                             timeout,
                             cancellable,
                             (GAsyncReadyCallback)end_call_ready,
                             task);
    qmi_message_unref (request);
}


QmiMessageVoiceAnswerCallOutput *
qmi_client_voice_answer_call_finish (
    QmiClientVoice *self,
    GAsyncResult *res,
    GError **error)
{
   return g_task_propagate_pointer (G_TASK (res), error);
}

static void
answer_call_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GTask *task)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageVoiceAnswerCallOutput *output;

    reply = qmi_device_command_full_finish (device, res, &error);
    if (!reply) {
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    /* Parse reply */
    output = __qmi_message_voice_answer_call_response_parse (reply, &error);
    if (!output)
        g_task_return_error (task, error);
    else
        g_task_return_pointer (task,
                               output,
                               (GDestroyNotify)qmi_message_voice_answer_call_output_unref);
    g_object_unref (task);
    qmi_message_unref (reply);
}

void
qmi_client_voice_answer_call (
    QmiClientVoice *self,
    QmiMessageVoiceAnswerCallInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GTask *task;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    task = g_task_new (self, cancellable, callback, user_data);
    if (!qmi_client_is_valid (QMI_CLIENT (self))) {
        g_task_return_new_error (task, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");
        g_object_unref (task);
        return;
    }

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_voice_answer_call_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    qmi_device_command_full (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                             request,
                             NULL,
                             timeout,
                             cancellable,
                             (GAsyncReadyCallback)answer_call_ready,
                             task);
    qmi_message_unref (request);
}


QmiMessageVoiceGetConfigOutput *
qmi_client_voice_get_config_finish (
    QmiClientVoice *self,
    GAsyncResult *res,
    GError **error)
{
   return g_task_propagate_pointer (G_TASK (res), error);
}

static void
get_config_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GTask *task)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageVoiceGetConfigOutput *output;

    reply = qmi_device_command_full_finish (device, res, &error);
    if (!reply) {
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    /* Parse reply */
    output = __qmi_message_voice_get_config_response_parse (reply, &error);
    if (!output)
        g_task_return_error (task, error);
    else
        g_task_return_pointer (task,
                               output,
                               (GDestroyNotify)qmi_message_voice_get_config_output_unref);
    g_object_unref (task);
    qmi_message_unref (reply);
}

void
qmi_client_voice_get_config (
    QmiClientVoice *self,
    QmiMessageVoiceGetConfigInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GTask *task;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    task = g_task_new (self, cancellable, callback, user_data);
    if (!qmi_client_is_valid (QMI_CLIENT (self))) {
        g_task_return_new_error (task, QMI_CORE_ERROR, QMI_CORE_ERROR_WRONG_STATE, "client invalid");
        g_object_unref (task);
        return;
    }

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_voice_get_config_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_task_return_error (task, error);
        g_object_unref (task);
        return;
    }

    qmi_device_command_full (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                             request,
                             NULL,
                             timeout,
                             cancellable,
                             (GAsyncReadyCallback)get_config_ready,
                             task);
    qmi_message_unref (request);
}

